﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using CaspitWebTiles.Api.Models;
using System.Windows.Forms;
using System.Web;
using System.ComponentModel;
using System.IO;

namespace ApiTester
{
    partial class frmMain
    {
        #region ///////////////// Single Document //////////////////////
        private void btnFindDocument_Click(object sender, EventArgs e)
        {
            try
            {

                string url = "";

                if (!string.IsNullOrEmpty(txtDocumentKey1.Text))
                {
                    url = string.Format("{0}/api/v1/documents/{2}?token={1}",
                     txtUrl.Text, txtToken.Text, txtDocumentKey1.Text);
                }
                else
                {
                    url = string.Format("{0}/api/v1/documents/?token={1}&Number={2}&trxTypeId={3}",
                     txtUrl.Text, txtToken.Text, txtDocumentNumber1.Text, lstDocumentTrxType2.SelectedValue);
                }

                string data = SendGetRequest(url);

                PopulateDocument(data);

                tabMain.SelectedTab = tabDocument;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }            
        }

        private void btnGetPDF_Click(object sender, EventArgs e)
        {
            try
            {

                if (string.IsNullOrEmpty(GetPDF_txtDocumentNumber.Text))
                {
                    MessageBox.Show("חובה להזין את המפתח של המסמך.", "", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                int lang = 0; //Hebrew
                if (GetPDF_lstLang.SelectedValue != null)
                {
                    lang = int.Parse(GetPDF_lstLang.SelectedValue.ToString());
                }

                string url = string.Format("{0}api/v1/PDF/?token={1}&id={2}&lan={3}",
                     txtUrl.Text, txtToken.Text, GetPDF_txtDocumentNumber.Text, lang);

                string data = SendGetRequest(url);

                tabMain.SelectedTab = tabPDF;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void PopulateDocument(string data)
        {

            try
            {
                CleanDocument();

                var doc = XDocument.Parse(data);
                XNamespace ns = "http://www.caspit.biz/2013/03"; //<== the NS is taken from the CaspitDocument declaration
                XElement root = doc.Root.Element(ns + "Results");

                if (root != null)
                {
                    //throw new ApplicationException("Unexpected result. Expected: CaspitDocument, Received: " + root.Name.LocalName);
                    int n = root.Elements(ns + "CaspitBase").Count();
                    root = root.Element(ns + "CaspitBase");
                    if (n > 1)
                    {
                        MessageBox.Show("Received a list of " + n.ToString() + " documents. Displaying the first document", "", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }
                else
                {
                    root = doc.Root;
                    if (doc.Root.Name.LocalName != "CaspitDocument")
                    {
                        MessageBox.Show("Could not find expected root elements in the reponse.\r\nResults or CaspitDocument", "", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }

                if (root == null)
                {
                    MessageBox.Show("No documents were found.");
                    return;
                }

                txtDocumentNumber.Text = root.Element(ns + "Number").Value;

                txtDocumentKey.Text = root.Element(ns + "DocumentId").Value;
                lstDocumentTrxType.SelectedValue = int.Parse(root.Element(ns + "TrxTypeId").Value);
                lstDocumentTrxCodes.SelectedValue = int.Parse(root.Element(ns + "TrxCodeNumber").Value);
                lstDocumentStatus.SelectedValue = int.Parse(root.Element(ns + "Status").Value);

                txtDocumentDate.Text = root.Element(ns + "Date").Value;

                txtDocumentClientForeignKey.Text = root.Element(ns + "CustomerId").Value;
                txtDocumentClientBusinessName.Text = root.Element(ns + "CustomerBusinessName").Value;
                txtDocumentClientOsekMorshe.Text = root.Element(ns + "CustomerOsekMorshe").Value;
                txtDocumentClientContactName.Text = root.Element(ns + "CustomerContactName").Value;
                txtDocumentClientAddress1.Text = root.Element(ns + "CustomerAddress1").Value;
                txtDocumentClientAddress2.Text = root.Element(ns + "CustomerAddress2").Value;
                txtDocumentClientCity.Text = root.Element(ns + "CustomerCity").Value;
                txtDocumentClientPostalCode.Text = root.Element(ns + "CustomerPostalCode").Value;

                txtDocumentComments.Text = root.Element(ns + "Comments").Value;
                txtDocumentDetails.Text = root.Element(ns + "Details").Value;

                txtDocumentDueDate.Text = root.Element(ns + "DueDate").Value;

                txtDocumentTotalLinesChargeVAT.Text = GetCValue(root, ns, "TotalLinesChargeVAT");
                txtDocumentRebate.Text = GetCValue(root, ns, "Rebate");
                chkDocumentRountTotal.Checked = bool.Parse(root.Element(ns + "RountTotal").Value);
                txtDocumentRebateRounding.Text = GetCValue(root, ns, "RebateRounding");

                txtDocumentTotalBeforeVATChargeVAT.Text = GetCValue(root, ns, "TotalBeforeVATChargeVAT");
                txtDocumentTotalNoChargeVat.Text = GetCValue(root, ns, "TotalNoChargeVat");
                txtDocumentTotal.Text = GetCValue(root, ns, "Total");
                txtDocumentVatRate.Text = GetCValue(root, ns, "VatRate");
                txtDocumentVat.Text = GetCValue(root, ns, "Vat");

                var documentLines = root.Descendants(ns + "CaspitDocumentLine");
                int i = 0;
                foreach (XElement line in documentLines)
                {
                    PopulateDocumentLine(line, ns, ++i);
                }

                var receiptLines = root.Descendants(ns + "CaspitReceiptLine");
                i = 0;
                foreach (XElement line in receiptLines)
                {
                    PopulateReceiptLine(line, ns, ++i);
                }

                txtTotalPaid1.Text = GetCValue(root, ns, "Payment");
                txtTaxDeduction.Text = GetCValue(root, ns, "TaxDeduction");
                txtTotalPaid.Text = GetCValue(root, ns, "TotalPayment");

                lstReceiptCurrency.SelectedValue = root.Element(ns + "ReceiptCurrencySymbol").Value;
                txtReceiptRate.Text = GetCValue(root, ns, "ReceiptRate");
                txtTotalPaidNIS.Text = GetCValue(root, ns, "TotalPaymentNIS");

                txtDocumentRowVersion.Text = root.Element(ns + "RowVersion").Value;

                txtLinkToDocument.Text = root.Element(ns + "LinkToPdf").Value;

                LinkLabel.Link link1 = new LinkLabel.Link();
                link1.LinkData = root.Element(ns + "ViewUrl").Value;
                lnkViewUrl.Links.Clear();
                lnkViewUrl.Links.Add(link1);
                lnkViewUrl.Text = root.Element(ns + "ViewUrl").Value;

                LinkLabel.Link link2 = new LinkLabel.Link();
                link2.LinkData = root.Element(ns + "PaymentUrl").Value;
                lnkPaymentUrl.Links.Clear();
                lnkPaymentUrl.Links.Add(link2);
                lnkPaymentUrl.Text = root.Element(ns + "PaymentUrl").Value;
            }
            catch 
            {
                throw;
            }
        }

        private void btnNewDocument_Click(object sender, EventArgs e)
        {
            try
            {
                string url;
                CaspitDocument document;

                PrepareDocumentMessage(out url, out document, txtDocumentKey.Text);

                string content = SendPostRequest<CaspitDocument>(url, document);

                PopulateDocument(content);

                MessageBox.Show("Operation completed OK. Data returned:\r\n" + content, "", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void btnUpdateDocument_Click(object sender, EventArgs e)
        {
            try
            {
                string url;
                CaspitDocument document;

                PrepareDocumentMessage(out url, out document, txtDocumentKey.Text);

                string content = SendPutRequest<CaspitDocument>(url, document);

                MessageBox.Show("Operation completed OK. Data returned:\r\n" + content, "", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnPrintDocument_Click(object sender, EventArgs e)
        {
            try
            {

                string url = "", fileName = "";
                int digitalSignature = tabPrintDocument_chkDigitalSignature.Checked ? 1 : 0;
                int lang = tabPrintDocument_lstLang.SelectedItem == "אנגלית" ? 1 : 0;

                if (!string.IsNullOrEmpty(txtPrintDocumentKey.Text))
                {
                    url = string.Format("{0}api/v1/documents/{2}?token={1}&lan={3}&createSignedPdf={4}",
                        txtUrl.Text, txtToken.Text, txtPrintDocumentKey.Text,
                        lang,
                        digitalSignature);
                    fileName = txtPrintDocumentKey.Text + ".pdf";
                }
                else
                {
                    url = string.Format("{0}api/v1/documents/?token={1}&DocumentNumber={2}&trxTypeId={3}&lan={4}&createSignedPdf={5}",
                     txtUrl.Text, txtToken.Text, txtPrintDocumentNumber.Text, 
                        lstPrintDocumentType.SelectedValue, 
                        lang,
                        digitalSignature);
                    fileName = txtPrintDocumentNumber.Text + ".pdf";
                }
                Encoding encoding = Encoding.GetEncoding(1252);
                string data = SendPostRequest<int?>(url, null, "PRINT", encoding: encoding);
                tabPrintDocument_txtPDFFileName.Text = SaveToFile(data, fileName.Replace('\\', '_').Replace('/', '_'));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        private void btnPrintDocumentPOST_Click(object sender, EventArgs e)
        {
            try
            {

                string url = "", fileName = "";
                int digitalSignature = tabPrintDocument_chkDigitalSignature.Checked ? 1 : 0;
                int lang = tabPrintDocument_lstLang.SelectedItem == "אנגלית" ? 1 : 0;

                if (!string.IsNullOrEmpty(txtPrintDocumentKey.Text))
                {
                    url = string.Format("{0}api/v1/printdocument/{2}?token={1}&lan={3}&createSignedPdf={4}",
                     txtUrl.Text, txtToken.Text, txtPrintDocumentKey.Text, lang, digitalSignature);
                    fileName = txtPrintDocumentKey.Text + ".pdf";
                }
                else
                {
                    url = string.Format("{0}api/v1/printdocument/?token={1}&DocumentNumber={2}&trxTypeId={3}&lan={4}&createSignedPdf={5}",
                     txtUrl.Text, txtToken.Text, txtPrintDocumentNumber.Text, lstPrintDocumentType.SelectedValue, lang, digitalSignature);
                    fileName = txtPrintDocumentNumber.Text + ".pdf";
                }
                Encoding encoding = Encoding.GetEncoding(1252);
                string data = SendPostRequest<int?>(url, null, encoding: encoding);
                tabPrintDocument_txtPDFFileName.Text = SaveToFile(data, fileName.Replace('\\', '_').Replace('/', '_'));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnSendDocumentEmail_Click(object sender, EventArgs e)
        {
            try
            {

                string url = "";

                if (!string.IsNullOrEmpty(txtDocumentEmailKey.Text))
                {
                    url = string.Format("{0}/api/v1/documents/{2}?token={1}",
                     txtUrl.Text, txtToken.Text, txtDocumentEmailKey.Text);
                }
                else if (!string.IsNullOrEmpty(txtDocumentEmailNumber.Text))
                {
                    url = string.Format("{0}/api/v1/documents/?token={1}&DocumentNumber={2}&trxTypeId={3}",
                     txtUrl.Text, txtToken.Text, txtDocumentEmailNumber.Text, lstDocumentTrxType1.SelectedValue);
                }
                else
                {
                    MessageBox.Show("חובה להכניס את מספר המסמך בכספית או את המפתח שלו במערכת שלך.", "", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                EmailDetails emailRequest = new EmailDetails();

                if (!string.IsNullOrEmpty(txtDocumentEmailTo.Text))
                {
                    emailRequest.To = txtDocumentEmailTo.Text;
                }
                if (!string.IsNullOrEmpty(txtDocumentEmailCC.Text))
                {
                    emailRequest.CC = txtDocumentEmailCC.Text;
                }
                if (!string.IsNullOrEmpty(txtDocumentEmailBCC.Text))
                {
                    emailRequest.BCC = txtDocumentEmailBCC.Text;
                }
                if (!string.IsNullOrEmpty(txtDocumentEmailSubject.Text))
                {
                    emailRequest.Subject = txtDocumentEmailSubject.Text;
                }
                if (!string.IsNullOrEmpty(txtDocumentEmailBody.Text))
                {
                    emailRequest.Body = txtDocumentEmailBody.Text;
                }
                string data = SendEmailRequest<EmailDetails>(url, emailRequest);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        private void btnGetDocument_Click(object sender, EventArgs e)
        {

        }

        void PopulateDocumentLine(XElement line, XNamespace ns, int number)
        {
            switch (number)
            {
                case 1:
                    txtDocumentLineNumber1.Text = line.Element(ns + "Number").Value;
                    txtDocumentLineDetails1.Text = line.Element(ns + "Details").Value;
                    txtDocumentLineQty1.Text = GetCValue(line, ns, "Qty");
                    txtDocumentLineUnitPrice1.Text = GetCValue(line, ns, "UnitPrice");
                    lstDocumentLineCurrency1.SelectedValue = line.Element(ns + "CurrencySymbol").Value;
                    txtDocumentLineRate1.Text = GetCValue(line, ns, "Rate");
                    txtDocumentLineRebate1.Text = GetCValue(line, ns, "Rebate");
                    txtDocumentLineTotal1.Text = string.Format("{0:#,##0.00}",
                        ((decimal.Parse(txtDocumentLineQty1.Text) *
                        decimal.Parse(txtDocumentLineUnitPrice1.Text) -
                        decimal.Parse(txtDocumentLineRebate1.Text)) *
                        decimal.Parse(txtDocumentLineRate1.Text)));
                    chkDocumentLineChargeVat1.Checked = bool.Parse(line.Element(ns + "ChargeVAT").Value);
                    lstDocumentLineProduct1.SelectedValue = line.Element(ns + "ProductId").Value;
                    break;
                case 2:
                    txtDocumentLineNumber2.Text = line.Element(ns + "Number").Value;
                    txtDocumentLineDetails2.Text = line.Element(ns + "Details").Value;
                    txtDocumentLineQty2.Text = GetCValue(line, ns, "Qty");
                    txtDocumentLineUnitPrice2.Text = GetCValue(line, ns, "UnitPrice");
                    lstDocumentLineCurrency2.SelectedValue = line.Element(ns + "CurrencySymbol").Value;
                    txtDocumentLineRate2.Text = GetCValue(line, ns, "Rate");
                    txtDocumentLineRebate2.Text = GetCValue(line, ns, "Rebate");
                    txtDocumentLineTotal2.Text = string.Format("{0:#,##0.00}",
                        ((decimal.Parse(txtDocumentLineQty2.Text) *
                        decimal.Parse(txtDocumentLineUnitPrice2.Text) -
                        decimal.Parse(txtDocumentLineRebate2.Text)) *
                        decimal.Parse(txtDocumentLineRate2.Text)));
                    chkDocumentLineChargeVat2.Checked = bool.Parse(line.Element(ns + "ChargeVAT").Value);
                    lstDocumentLineProduct2.SelectedValue = line.Element(ns + "ProductId").Value;
                    break;
                case 3:
                    txtDocumentLineNumber3.Text = line.Element(ns + "Number").Value;
                    txtDocumentLineDetails3.Text = line.Element(ns + "Details").Value;
                    txtDocumentLineQty3.Text = GetCValue(line, ns, "Qty");
                    txtDocumentLineUnitPrice3.Text = GetCValue(line, ns, "UnitPrice");
                    lstDocumentLineCurrency3.SelectedValue = line.Element(ns + "CurrencySymbol").Value;
                    txtDocumentLineRate3.Text = GetCValue(line, ns, "Rate");
                    txtDocumentLineRebate3.Text = GetCValue(line, ns, "Rebate");
                    txtDocumentLineTotal3.Text = string.Format("{0:#,##0.00}",
                        ((decimal.Parse(txtDocumentLineQty3.Text) *
                        decimal.Parse(txtDocumentLineUnitPrice3.Text) -
                        decimal.Parse(txtDocumentLineRebate3.Text)) *
                        decimal.Parse(txtDocumentLineRate3.Text)));
                    chkDocumentLineChargeVat3.Checked = bool.Parse(line.Element(ns + "ChargeVAT").Value);
                    lstDocumentLineProduct3.SelectedValue = line.Element(ns + "ProductId").Value;
                    break;
            }
        }
        void PopulateReceiptLine(XElement line, XNamespace ns, int number)
        {
            switch (number)
            {
                case 1:
                    txtReceiptLineNumber1.Text = line.Element(ns + "Number").Value;
                    lstReceiptLinePmtType1.SelectedValue = int.Parse(line.Element(ns + "PaymentTypeId").Value);
                    txtReceiptLineAcctNum1.Text = line.Element(ns + "AccountNumber").Value;
                    txtReceiptLineBranchNum1.Text = line.Element(ns + "BranchOrValidTo").Value;
                    lstReceiptLineBank1.SelectedValue = line.Element(ns + "BankId").Value;
                    txtReceiptLineCheckNum1.Text = line.Element(ns + "CheckNumber").Value;
                    txtReceiptLineDate1.Text = line.Element(ns + "PaymentDate").Value;
                    txtReceiptLineAmount1.Text = GetCValue(line, ns, "Payment");
                    break;
                case 2:
                    txtReceiptLineNumber2.Text = line.Element(ns + "Number").Value;
                    lstReceiptLinePmtType2.SelectedValue = int.Parse(line.Element(ns + "PaymentTypeId").Value);
                    txtReceiptLineAcctNum2.Text = line.Element(ns + "AccountNumber").Value;
                    txtReceiptLineBranchNum2.Text = line.Element(ns + "BranchOrValidTo").Value;
                    lstReceiptLineBank2.SelectedValue = line.Element(ns + "BankId").Value;
                    txtReceiptLineCheckNum2.Text = line.Element(ns + "CheckNumber").Value;
                    txtReceiptLineDate2.Text = line.Element(ns + "PaymentDate").Value;
                    txtReceiptLineAmount2.Text = GetCValue(line, ns, "Payment");
                    break;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="url"></param>
        /// <param name="document"></param>
        /// <param name="id">The id WE (=the client) use to identify the document. It is NOT the document number in Caspit.</param>
        private void PrepareDocumentMessage(out string url, out CaspitDocument document, string id)
        {
            //  .../api/v1/documents/2a193514-e412-21b6-a97b-5f0ae26ad332?token=4b193514-e412-41b7-a97b-5f0ae26ad351
            url = string.Format("{0}/api/v1/documents/{2}?token={1}",
                txtUrl.Text, txtToken.Text, id);

            if (lstReceiptCurrency.SelectedItem == null)
            {
                throw new Exception("חובה לבחור במטבע בשורת הקבלה");
            }

            document = new CaspitDocument
            {               
                DocumentId = txtDocumentKey.Text,
                Date = DateTime.Parse(txtDocumentDate.Text),
                CustomerId = txtDocumentClientForeignKey.Text,
                CustomerBusinessName = txtDocumentClientBusinessName.Text,
                CustomerContactName = txtDocumentClientContactName.Text,
                CustomerOsekMorshe = txtDocumentClientOsekMorshe.Text,
                CustomerAddress1 = txtDocumentClientAddress1.Text,
                CustomerAddress2 = txtDocumentClientAddress2.Text,
                CustomerCity = txtDocumentClientCity.Text,
                CustomerPostalCode = txtDocumentClientPostalCode.Text,
                CustomerEmail = Z(txtMail.Text),
                //
                Rebate = string.IsNullOrEmpty(txtDocumentRebate.Text) ? 0 : decimal.Parse(txtDocumentRebate.Text),
                RountTotal = chkDocumentRountTotal.Checked,
                Details = txtDocumentDetails.Text,
                Comments = txtDocumentComments.Text,
                DueDate = string.IsNullOrEmpty(txtDocumentDueDate.Text) ? (DateTime?)null : DateTime.Parse(txtDocumentDueDate.Text),
                //Receipt
                ReceiptCurrencySymbol = ((ValueNamePair<string>)lstReceiptCurrency.SelectedItem).Value,
                ReceiptRate = string.IsNullOrEmpty(txtReceiptRate.Text) ? 0 : decimal.Parse(txtReceiptRate.Text),
                TaxDeduction = string.IsNullOrEmpty(txtTaxDeduction.Text) ? 0 : decimal.Parse(txtTaxDeduction.Text),
                TrxCodeNumber = int.Parse(lstTrxCodes.SelectedValue.ToString()),
                TrxTypeId = int.Parse(lstDocumentTrxType.SelectedValue.ToString()),
                RowVersion = Convert.FromBase64String(txtDocumentRowVersion.Text)
            };

            document.DocumentLines = new List<CaspitDocumentLine>();

            document.DocumentLines.Add(GetDocumentLine(1));
            document.DocumentLines.Add(GetDocumentLine(2));
            document.DocumentLines.Add(GetDocumentLine(3));

            document.ReceiptLines = new List<CaspitReceiptLine>();

            document.ReceiptLines.Add(GetReceiptLine(1));
            document.ReceiptLines.Add(GetReceiptLine(2));

        }

        CaspitDocumentLine GetDocumentLine(int n)
        {
            CaspitDocumentLine line = null;

            switch (n)
            {
                case 1:
                    if (lstDocumentLineProduct1.SelectedIndex > 0 || !string.IsNullOrEmpty(txtDocumentLineDetails1.Text))
                    {
                        line = new CaspitDocumentLine
                        {
                            Number = 1,
                            ProductId = (lstDocumentLineProduct1.SelectedValue ?? "").ToString(),
                            Details = txtDocumentLineDetails1.Text,
                            Qty = decimal.Parse(txtDocumentLineQty1.Text),
                            UnitPrice = decimal.Parse(txtDocumentLineUnitPrice1.Text),
                            CurrencySymbol = (lstDocumentLineCurrency1.SelectedValue ?? "").ToString(),
                            Rate = decimal.Parse(txtDocumentLineRate1.Text),
                            Rebate = decimal.Parse(txtDocumentLineRebate1.Text),
                            ChargeVAT = chkDocumentLineChargeVat1.Checked
                        };
                    }
                    break;
                case 2:
                    if (lstDocumentLineProduct2.SelectedIndex > 0 || !string.IsNullOrEmpty(txtDocumentLineDetails2.Text))
                    {
                        line = new CaspitDocumentLine
                        {
                            Number = 2,
                            ProductId = (lstDocumentLineProduct2.SelectedValue ?? "").ToString(),
                            Details = txtDocumentLineDetails2.Text,
                            Qty = decimal.Parse(txtDocumentLineQty2.Text),
                            UnitPrice = decimal.Parse(txtDocumentLineUnitPrice2.Text),
                            CurrencySymbol = (lstDocumentLineCurrency2.SelectedValue ?? "").ToString(),
                            Rate = decimal.Parse(txtDocumentLineRate2.Text),
                            Rebate = decimal.Parse(txtDocumentLineRebate2.Text),
                            ChargeVAT = chkDocumentLineChargeVat2.Checked
                        };
                    }
                    break;
                case 3:
                    if (lstDocumentLineProduct3.SelectedIndex > 0 || !string.IsNullOrEmpty(txtDocumentLineDetails3.Text))
                    {
                        line = new CaspitDocumentLine
                        {
                            Number = 3,
                            ProductId = (lstDocumentLineProduct3.SelectedValue ?? "").ToString(),
                            Details = txtDocumentLineDetails3.Text,
                            Qty = decimal.Parse(txtDocumentLineQty3.Text),
                            UnitPrice = decimal.Parse(txtDocumentLineUnitPrice3.Text),
                            CurrencySymbol = (lstDocumentLineCurrency3.SelectedValue ?? "").ToString(),
                            Rate = decimal.Parse(txtDocumentLineRate3.Text),
                            Rebate = decimal.Parse(txtDocumentLineRebate3.Text),
                            ChargeVAT = chkDocumentLineChargeVat3.Checked
                        };
                    }
                    break;
            }

            return line;
        }

        CaspitReceiptLine GetReceiptLine(int n)
        {
            CaspitReceiptLine line = null;

            switch (n)
            {
                case 1:
                    if (lstReceiptLinePmtType1.SelectedIndex != 0)
                    {
                        if (lstReceiptLinePmtType1.SelectedValue == null) throw new Exception("Invalid PaymentType1");
                        line = new CaspitReceiptLine
                        {
                            Number = 1,
                            AccountNumber = txtReceiptLineAcctNum1.Text,
                            BankId = lstReceiptLineBank1.SelectedValue == null ? (int?)null : int.Parse(lstReceiptLineBank1.SelectedValue.ToString()),
                            BranchOrValidTo = txtReceiptLineBranchNum1.Text,
                            CheckNumber = txtReceiptLineCheckNum1.Text,
                            Payment = decimal.Parse(txtReceiptLineAmount1.Text),
                            PaymentDate = string.IsNullOrEmpty(txtReceiptLineDate1.Text) ? (DateTime?)null : DateTime.Parse(txtReceiptLineDate1.Text),
                            PaymentTypeId = int.Parse(lstReceiptLinePmtType1.SelectedValue.ToString()),
                        };
                    }
                    break;
                case 2:
                    if (lstReceiptLinePmtType2.SelectedIndex != 0)
                    {
                        if (lstReceiptLinePmtType2.SelectedValue == null) throw new Exception("Invalid PaymentType2");
                        line = new CaspitReceiptLine
                        {
                            Number = 2,
                            AccountNumber = txtReceiptLineAcctNum2.Text,
                            BankId = lstReceiptLineBank2.SelectedValue == null ? (int?)null : int.Parse(lstReceiptLineBank2.SelectedValue.ToString()),
                            BranchOrValidTo = txtReceiptLineBranchNum2.Text,
                            CheckNumber = txtReceiptLineCheckNum2.Text,
                            Payment = decimal.Parse(txtReceiptLineAmount2.Text),
                            PaymentDate = string.IsNullOrEmpty(txtReceiptLineDate2.Text) ? (DateTime?)null : DateTime.Parse(txtReceiptLineDate2.Text),
                            PaymentTypeId = int.Parse(lstReceiptLinePmtType2.SelectedValue.ToString()),
                        };
                    }
                    break;
            }
            return line;
        }
        private void btnCleanDocument_Click(object sender, EventArgs e)
        {
            CleanDocument();
        }

        private void CleanDocument()
        {
            txtDocumentKey.Text = null;

            lstDocumentTrxType.SelectedValue = 0;
            lstDocumentTrxCodes.SelectedValue = 0;

            txtDocumentDate.Text = null;

            txtDocumentClientForeignKey.Text = null;
            txtDocumentClientBusinessName.Text = null;
            txtDocumentClientOsekMorshe.Text = null;
            txtDocumentClientContactName.Text = null;
            txtDocumentClientAddress1.Text = null;
            txtDocumentClientAddress2.Text = null;
            txtDocumentClientCity.Text = null;
            txtDocumentClientPostalCode.Text = null;

            txtDocumentComments.Text = null;
            txtDocumentDetails.Text = null;

            txtDocumentDueDate.Text = null;

            txtDocumentTotalLinesChargeVAT.Text = null;
            txtDocumentRebate.Text = null;
            chkDocumentRountTotal.Checked = false;
            txtDocumentRebateRounding.Text = null;

            txtDocumentTotalBeforeVATChargeVAT.Text = null;
            txtDocumentTotalNoChargeVat.Text = null;
            txtDocumentTotal.Text = null;
            txtDocumentVatRate.Text = null;
            txtDocumentVat.Text = null;

            txtDocumentLineNumber1.Text = null;
            txtDocumentLineDetails1.Text = null;
            txtDocumentLineQty1.Text = null;
            txtDocumentLineUnitPrice1.Text = null;
            lstDocumentLineCurrency1.SelectedIndex = 0;
            txtDocumentLineRate1.Text = null;
            txtDocumentLineRebate1.Text = null;
            txtDocumentLineTotal1.Text = null;
            chkDocumentLineChargeVat1.Checked = false;

            txtDocumentLineNumber2.Text = null;
            txtDocumentLineDetails2.Text = null;
            txtDocumentLineQty2.Text = null;
            txtDocumentLineUnitPrice2.Text = null;
            lstDocumentLineCurrency2.SelectedIndex = 0;
            txtDocumentLineRate2.Text = null;
            txtDocumentLineRebate2.Text = null;
            txtDocumentLineTotal2.Text = null;
            chkDocumentLineChargeVat2.Checked = false;

            txtDocumentLineNumber3.Text = null;
            txtDocumentLineDetails3.Text = null;
            txtDocumentLineQty3.Text = null;
            txtDocumentLineUnitPrice3.Text = null;
            lstDocumentLineCurrency3.SelectedIndex = 0;
            txtDocumentLineRate3.Text = null;
            txtDocumentLineRebate3.Text = null;
            txtDocumentLineTotal3.Text = null;
            chkDocumentLineChargeVat3.Checked = false;

            //Receipt
            txtReceiptLineNumber1.Text = null;
            lstReceiptLinePmtType1.SelectedIndex = 0;
            txtReceiptLineAcctNum1.Text = null;
            txtReceiptLineBranchNum1.Text = null;
            lstReceiptLineBank1.SelectedIndex = 0;
            txtReceiptLineCheckNum1.Text = null;
            txtReceiptLineDate1.Text = null;
            txtReceiptLineAmount1.Text = null;

            txtReceiptLineNumber2.Text = null;
            lstReceiptLinePmtType2.SelectedIndex = 0;
            txtReceiptLineAcctNum2.Text = null;
            txtReceiptLineBranchNum2.Text = null;
            lstReceiptLineBank2.SelectedIndex = 0;
            txtReceiptLineCheckNum2.Text = null;
            txtReceiptLineDate2.Text = null;
            txtReceiptLineAmount2.Text = null;

            txtTaxDeduction.Text = null;
            txtTotalPaid1.Text = null;
            txtTotalPaid.Text = null;

            lstReceiptCurrency.SelectedIndex = 0;
            txtReceiptRate.Text = null;
            txtTotalPaidNIS.Text = null;

        }

        private void tabActions_cmdChargeAccount_Click(object sender, EventArgs e)
        {
            try
            {
                string url = string.Format("{0}api/v1/ChargeAccount/?token={1}", txtUrl.Text, txtToken.Text);
                if (string.IsNullOrEmpty(tabActions_AccountNumber.Text))
                {
                    MessageBox.Show("חובה להזין את המספר של הח-ן .", "", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                string content = SendPostRequest<int>(url, int.Parse(tabActions_AccountNumber.Text));
                MessageBox.Show("Operation completed OK. Data returned:\r\n" + content, "", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// The Currency symbols are taken from Caspit currency table
        /// You can see the values in מטבעות screen
        /// </summary>
        /// <param name="c"></param>
        private void InitCurrencyList(ComboBox c)
        {
            List<ValueNamePair<string>> v4 = new List<ValueNamePair<string>>();
            v4.Add(new ValueNamePair<string>(null, "--בחר--"));
            v4.Add(new ValueNamePair<string>("₪", "ש\"ח"));
            v4.Add(new ValueNamePair<string>("$", "$"));
            v4.Add(new ValueNamePair<string>("€", "יורו"));
            c.DataSource = v4;
            c.DisplayMember = "Name";
            c.ValueMember = "Value";
            c.SelectedIndex = 0;
        }

        private void InitLangList(ComboBox c)
        {
            List<ValueNamePair<string>> v = new List<ValueNamePair<string>>();
            v.Add(new ValueNamePair<string>(null, "--בחר--"));
            v.Add(new ValueNamePair<string>("0", "עברית"));
            v.Add(new ValueNamePair<string>("1", "English"));
            c.DataSource = v;
            c.DisplayMember = "Name";
            c.ValueMember = "Value";
            c.SelectedIndex = 0;
        }
        /// <summary>
        /// The bank numbers are defined by Bank Israel
        /// </summary>
        /// <param name="c"></param>
        private void InitBankList(ComboBox c)
        {
            List<ValueNamePair<int?>> v = new List<ValueNamePair<int?>>();
            v.Add(new ValueNamePair<int?>(null, "--בחר--"));
            v.Add(new ValueNamePair<int?>(4, "יהב"));
            v.Add(new ValueNamePair<int?>(10, "לאומי"));
            v.Add(new ValueNamePair<int?>(11, "דיסקונט"));
            v.Add(new ValueNamePair<int?>(12, "הפועלים"));
            v.Add(new ValueNamePair<int?>(52, "פאי"));

            c.DataSource = v;
            c.DisplayMember = "Name";
            c.ValueMember = "Value";
            c.SelectedIndex = 0;
        }

        /// <summary>
        /// Values takem from ApiGlobals.PaymentTypes
        /// </summary>
        /// <param name="c"></param>
        private void InitPmtTypeList(ComboBox c)
        {
            List<ValueNamePair<int?>> v = new List<ValueNamePair<int?>>();
            v.Add(new ValueNamePair<int?>(null, "--בחר--"));
            v.Add(new ValueNamePair<int?>(1, "מזומן"));
            v.Add(new ValueNamePair<int?>(2, "שיק"));
            v.Add(new ValueNamePair<int?>(3, "ויזה כאל"));
            v.Add(new ValueNamePair<int?>(4, "ויזה לאומי"));
            v.Add(new ValueNamePair<int?>(6, "מאסטרכרד"));
            v.Add(new ValueNamePair<int?>(9, "העברה"));
            v.Add(new ValueNamePair<int?>(14, "אינטרנט"));

            c.DataSource = v;
            c.DisplayMember = "Name";
            c.ValueMember = "Value";
            c.SelectedIndex = 0;
        }

        /// <summary>
        /// Values taken from ApiGlobals.DocStatus
        /// </summary>
        /// <param name="c"></param>
        private void InitDocumentStatusList(ComboBox c)
        {
            List<ValueNamePair<int?>> v = new List<ValueNamePair<int?>>();
            v.Add(new ValueNamePair<int?>(null, "--בחר--"));
            v.Add(new ValueNamePair<int?>(0, "חדש"));
            v.Add(new ValueNamePair<int?>(2, "הודפס"));
            v.Add(new ValueNamePair<int?>(3, "בוטל"));

            c.DataSource = v;
            c.DisplayMember = "Name";
            c.ValueMember = "Value";
            c.SelectedIndex = 0;
        }

        public string SaveToFile(string data, string fileName)
        {
            System.IO.FileStream _FileStream = new System.IO.FileStream(fileName, System.IO.FileMode.Create,
                                        System.IO.FileAccess.Write);
            Encoding encoding = Encoding.GetEncoding(1252);
            byte[] _ByteArray = encoding.GetBytes(data);

            _FileStream.Write(_ByteArray, 0, _ByteArray.Length);

            _FileStream.Close();

            return _FileStream.Name;
        }



        //private void SafePDF(string data, string fileName)
        //{
        //    Stream sourceStream = GenerateStreamFromString(data); // whereever the source stream is
        //    FileStream targetStream = File.OpenWrite(fileName);
        //    int bytesRead = 0;
        //    byte[] buffer = new byte[2048];
        //    while (true)
        //    {
        //        bytesRead = sourceStream.Read(buffer, 0, buffer.Length);
        //        if (bytesRead == 0)
        //            break;
        //        targetStream.Write(buffer, 0, bytesRead);
        //    }
        //    sourceStream.Close();
        //    targetStream.Close();
        //}
        //private static Stream GenerateStreamFromString(string s)
        //{
        //    MemoryStream stream = new MemoryStream();
        //    StreamWriter writer = new StreamWriter(stream);
        //    writer.Write(s);
        //    writer.Flush();
        //    stream.Position = 0;
        //    return stream;
        //}
        #endregion
    }
}
