﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CaspitWebTiles.Api
{
    public class ServerCommonGlobals
    {
        /// <summary>
        /// 
        /// </summary>
        public enum TrxTypes
        {
            //Caspit internal note: Do not change these values since they are encoded into the SystemConfigurations values (e.f. Commentחשבונית)
            חשבונית = 1,
            קבלה = 2,
            הוצאה = 3,
            /// <summary>
            /// 4 -- חשבונית ידנית
            /// </summary>
            חשבונית_ידנית = 4,
            /// <summary>
            /// תשלום עבור הוצאה
            /// </summary>
            תשלום_עבור_הוצאה = 5,
            חשבונית_עסקה = 6,
            /// <summary>
            /// 7 -- חש/קבלה - החשבונית
            /// </summary>
            חשבונית_קבלה = 7,
            /// <summary>
            /// 8 -- חש/קבלה - חלק הקבלה (שימוש פנימי של כספית) 
            /// </summary>
            חש_קבלה_הקבלה = 8,
            חשבונית_זיכוי = 9,
            מסמך_מבוטל = 10,
            קבלה_לתרומה_ידנית = 11,
            קבלה_ידנית = 12,
            /// <summary>
            /// תעודת החזר כספים
            /// </summary>
            החזר_כספים = 13,
            הזמנת_עבודה = 14,
            תעודת_משלוח = 15,
            /// <summary>
            /// הצעת מחיר
            /// </summary>
            הצעת_מחיר = 16,
            /// <summary>
            /// ספירת מלאי
            /// </summary>
            ספירת_מלאי = 17,
            /// <summary>
            /// חשבונית או ת"מ של הספק
            /// </summary>
            מסמך_של_ספק = 18,
            קבלה_לתרומה = 19,
            /// <summary>
            /// תעודת תשלום מפיקדון
            /// </summary>
            פיקדון = 23,
            /// <summary>
            /// חשבון (כספית+עמותות)
            /// </summary>
            חשבון_עמותות = 30,
            /// <summary>
            /// חשבון/קבלה -- החשבון
            /// </summary>
            חשבון_קבלה = 31,
            /// <summary>
            /// חשבון/קבלה -- הקבלה
            /// </summary>
            חשבון_קבלה_הקבלה = 33,
            חשבון_זיכוי = 34,
            הזמנת_רכש = 47,
            /// <summary>
            /// יתרת פתיחה
            /// </summary>
            יתרת_פתיחה = 48,

            /// <summary>
            /// שכר טרחה (כספית עורכי דין)
            /// </summary>
            שכט = 20,
            /// <summary>
            /// שכ"ט נוטריוני
            /// </summary>
            שכט_נוטריוני = 21,
            /// <summary>
            /// תשלום לפיקדון
            /// </summary>
            תשלום_לפיקדון = 22,
            /// <summary>
            /// חיוב לקוח בהוצאות ללא מע"מ
            /// </summary>
            Expenses_No_VAT = 24,
            /// <summary>
            /// חיוב לקוח בהוצאות עם מע"מ
            /// </summary>
            Expenses_VAT = 33,
            /// <summary>
            /// שונות
            /// </summary>
            שונות = 25
        }

        /// <summary>
        /// הסטטוס של מסמך בכספית
        /// </summary>
        public enum DocStatus
        {
            NewDoc = 0,
            Printed = 2,
            Canceled = 3,
            //A special status for PriceQuotes: Both Printed and still can be updated
            PrintedUpdatable = 4
        }

        public enum ContactStatus
        {
            Inactive = 0,
            Active = 1,
            ActiveMizdamen = 2
        }

        /// <summary>
        /// הסוג של אנשי הקשר
        /// </summary>
        public enum ContactType
        {
            Customer = 0,
            Supplier = 1,
            CustomerAndSupplier = 2
        }


        public enum SupplierType
        {
            חד_פעמי = 0,
            מחייב_חודשי = 1,
            מחייב_דו_חודשי_זוגי = 3,
            מחייב_דו_חודשי_אי_זוגי = 4
        }

        public enum ProductStatus
        {
            Inactive = 0,
            Active = 1
        }

        /// <summary>
        /// סוגי התנועות (=מסמכים) בכספית
        /// </summary>
        public enum TrxCodes
        {
            קבלה = 2,
            הכנסות_מכירות = 3,
            הכנסות_פטורות_ממעמ = 4,
            קניית_רכוש_קבוע_מוכר_למעמ = 21,
            קניית_רכוש_קבוע_לא_מוכר_למעמ = 22,
            מכירת_רכוש_קבוע_חייב_במעמ = 23,
            מכירת_רכוש_קבוע_פטור_ממעמ = 24,
            הזמנות_רכש = 50
        }

        public enum PLCodes
        {
            /// <summary>
            /// הכנסות/מכירות
            /// </summary>
            הכנסות_מכירות = 1,
            /// <summary>
            /// הכנסות מפעולות אחרות
            /// </summary>
            הכנסות_מפעולות_אחרות = 4,
            /// <summary>
            /// תצרוכת חומרים
            /// </summary>
            תצרוכת_חומרים = 2,
            /// <summary>
            /// הוצאות מכירה, הנהלה וכלליות
            /// </summary>
            הוצאות_מכירה_הנהלה_וכלליות = 3,
            /// <summary>
            /// הוצאות מפעולות אחרות
            /// </summary>
            הוצאות_מפעולות_אחרות = 5,
            /// <summary>
            /// הוצאות מימון
            /// </summary>
            הוצאות_מימון = 6,
            /// <summary>
            /// פיקדונות
            /// </summary>
            Deposits = 7,
            קבלות = 8,
            /// <summary>
            /// לא משתתף
            /// </summary>
            לא_משתתף = 9
        }

        /// <summary>
        /// אמצעי תשלום בשורת קבלה
        /// </summary>
        public enum PaymentTypes
        {
            מזומן = 1,
            שיק = 2,
            ויזה_כאל = 3,
            ויזה_לאומי = 4,
            ישראכרד = 5,
            מסטרכארד = 6,
            אמקס = 7,
            כא_אחר = 8,
            העברה_בנקאית = 9,
            הוראת_קבע = 10,
            תשלום_מפיקדון = 11,
            אחר = 12,
            שובר_דואר = 13,
            אינטרנט = 14,
            PayPal = 15
        }

        public enum ErrorNumber
        {
            Document_Receipt_Part_Not_Equal_Invoice_Part = 10001,
            Business_Is_Locked = 10002,
            No_Document_In_System = 10003
        }

        public enum HelpType
        {
            Help = 1,
            Faq = 2,
            UsageTip = 3,
            AccountingTip  = 4
        }
    }
}
