﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace CaspitWebTiles.Api.Models
{
    [DataContract(Namespace = "http://www.caspit.biz/2013/03")]
    public class CaspitCreditCard : CaspitBase
    {
        /// <summary>
        /// The credit card's number
        /// </summary>
        [DataMember, StringLength(60)]
        public virtual string CCNumber { get; set; }

        /// <summary>
        /// The token used to identify the credit card
        /// </summary>
        [DataMember, StringLength(60)]
        public virtual string CCNumberToken { get; set; }

        /// <summary>
        /// The credit card's holder ID number (called TZ in Israel)
        /// </summary>
        [DataMember, StringLength(9, ErrorMessage = "שדה מספר ת\"ז של בעל הכרטיס מוגבל ל-9 תווים")]
        public virtual string CCHolderTZ { get; set; }

        /// <summary>
        /// Expiration date in the format "1223" for 12/2023
        /// </summary>
        [DataMember, RegularExpression("^(0[1-9]|1[0-2])\\/?([0-9]{4}|[0-9]{2})$", ErrorMessage = "תוקף הכרטיס לא תקין. הכנס התוקף בפורמט MMYY")]
        [StringLength(4, ErrorMessage = "שדה תוקף הכרטיס מוגבל ל-4 תווים: MMYY")]
        public virtual string CCExpDate { get; set; }

        /// <summary>
        /// CVV2
        /// </summary>
        [DataMember, StringLength(4)]
        public virtual string CVV { get; set; }
    }
}
