﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace CaspitWebTiles.Api.Models
{
    /// <summary>
    /// The details for charging a credit card
    /// </summary>
    [DataContract(Namespace = "http://www.caspit.biz/2013/03")]
    public class CaspitCreditCharge : CaspitBase
    {
        /// <summary>
        /// The credit card's number
        /// </summary>
        [DataMember, StringLength(60)]
        public virtual string CCNumber { get; set; }
        
        /// <summary>
        /// The token used to identify the credit card
        /// </summary>
        [DataMember, StringLength(60)]
        public virtual string CCNumberToken { get; set; }

        /// <summary>
        /// The credit card's holder ID number (called TZ in Israel)
        /// </summary>
        [DataMember, StringLength(9, ErrorMessage = "שדה מספר ת\"ז של בעל הכרטיס מוגבל ל-9 תווים")]
        public virtual string CCHolderTZ { get; set; }

        /// <summary>
        /// Expiration date in the format "0223" for 2/2023
        /// </summary>
        [DataMember, RegularExpression("^(0[1-9]|1[0-2])\\/?([0-9]{4}|[0-9]{2})$", ErrorMessage = "תוקף הכרטיס לא תקין. הכנס התוקף בפורמט MMYY")]
        [StringLength(4, ErrorMessage = "שדה תוקף הכרטיס מוגבל ל-4 תווים: MMYY")]
        public virtual string CCExpDate { get; set; }

        /// <summary>
        /// CVV2
        /// </summary>
        [DataMember, StringLength(4)]
        public virtual string CVV { get; set; }

        /// <summary>
        /// The amount to be charged
        /// </summary>
        [DataMember]
        public virtual decimal Amount { get; set; }

        /// <summary>
        /// סמל המטבע - הסמלים האפשריים הם: ₪, $
        /// </summary>
        [DataMember, StringLength(4), DefaultValue("₪")]
        public string CurrencySymbol { get; set; }

        /// <summary>
        /// Possible values: 1 = No payments (= charge the amount in 1 payment), 6 = "Credit", 8 = Payments
        /// The values are based on Shva's AshraitCreditTypes
        /// </summary>
        [DataMember, DefaultValue(1)]
        public virtual int CreditType { get; set; }

        /// <summary>
        /// The number of payments.
        /// If CreditType == 1 then NumPayments must be 1
        /// If CreditType == 6 or 8 then NumPayments must be larger than 1
        /// </summary>
        [DataMember, DefaultValue(1)]
        public virtual int NumPayments { get; set; }

        /// <summary>
        /// Is this charge a Refund?
        /// </summary>
        [DataMember, DefaultValue(false)]
        public bool IsRefund { get; set; }
    }
}
