﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace CaspitWebTiles.Api.Models
{
    /// <summary>
    /// Represents an expense
    /// </summary>
    [DataContract(Namespace = "http://www.caspit.biz/2013/03")]
    public class CaspitExpense : CaspitBase
    {
        /// <summary>
        /// מספר התנועה
        /// </summary>
        [Key, Required, DataMember]
        public int Number { get; set; }
        
        /// <summary>
        /// מספר המנה
        /// </summary>
        [DataMember]
        public int PortionNumber { get; set; }

        /// <summary>
        /// תאריך ההוצאה
        /// </summary>
        [Required, DataMember]
        public DateTime Date { get; set; }

        /// <summary>
        /// שמו של סיווג תנועה של המסמך. לדוגמה: "אחזקת משרד"
        /// </summary>
        [DataMember, StringLength(50)]
        public string TrxCode { get; set; }
        [Required, DataMember]
        public int TrxCodeNumber { get; set; }

        /// <summary>
        /// מספר עוסק מורשה של הספק
        /// </summary>
        [DataMember, StringLength(20)]
        public  string OsekMorshe { get; set; }
        /// <summary>
        /// שם הספק
        /// </summary>
        [DataMember, StringLength(200)]
        public string SupplierName { get; set; }
        /// <summary>
        /// המפתח של הספק
        /// </summary>
        [Required, DataMember, StringLength(50)]
        public string SupplierId { get; set; }

        /// <summary>
        /// פרטי ההוצאה
        /// </summary>
        [DataMember, StringLength(400)]
        public string Details { get; set; }

        /// <summary>
        /// אסמכתא = מספר החשבונית של הספק
        /// </summary>
        [DataMember, StringLength(50)]
        public string Reference { get; set; }

        /// <summary>
        /// סה"כ כולל מע"מ בש"ח
        /// </summary>
        [DataMember]
        public decimal Total { get; set; }
        /// <summary>
        /// סכום ההוצאה לפני מע"מ בש"ח
        /// </summary>
        [DataMember]
        public decimal TotalNoVat { get; set; }
        /// <summary>
        /// מע"מ בש"ח
        /// </summary>
        [DataMember]
        public decimal Vat { get; set; }

        /// <summary>
        /// אחוז המעמ: 16.00 עבור 16%
        /// </summary>
        [DataMember, Range(0.00, 100.001)]
        public decimal VatRate { get; set; }

        /// <summary>
        /// תשלומים עבור ההוצאה
        /// </summary>
        [DataMember]
        public List<CaspitExpensePayment> Payments;
    }
}
