﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.DataAnnotations;
using System.Runtime.Serialization;

namespace CaspitWebTiles.Api.Models
{
    /// <summary>
    /// A class for a Product
    /// </summary>
    [DataContract(Namespace = "http://www.caspit.biz/2013/03")]
    public class CaspitProduct : CaspitBase
    {
        /// <summary>
        /// The product's Id. You need to supply a unique Id that you will use to fetch or update the product.
        /// </summary>
        [Key, Required, DataMember, StringLength(50)]
        public virtual string ProductId { get; set; }

        /// <summary>
        /// שם הפריט, עד 255 תווים
        /// </summary>
        [DataMember, StringLength(255)]
        public virtual string Name { get; set; }
        /// <summary>
        /// תיאור הפריט - התיאור משולב עם שם הפריט ומוצג בחשבונית
        /// </summary>
        [DataMember, StringLength(4000)]
        public virtual string Description { get; set; }
        /// <summary>
        /// שם הפריט באנגלית, עד 255 תווים
        /// </summary>
        [DataMember, StringLength(255)]
        public virtual string NameEnglish { get; set; }

        /// <summary>
        /// English description. Used in English documents, 4000 chars
        /// </summary>
        [DataMember, StringLength(4000)]
        public virtual string DescriptionEnglish { get; set; }

        /// <summary>
        /// הערות למוצר. לא מוצגות ללקוח. עד 4000 תווים
        /// </summary>
        [DataMember, StringLength(4000)]
        public virtual string Comments { get; set; }

        /// <summary>
        /// מספר סידורי שמוטבע על הפריט - מיועד לפריטים יקרים שמזוהים באופן יחידני, עד 40 תווים
        /// </summary>
        [DataMember, StringLength(40)]
        public virtual string SerialNumber { get; set; }

        /// <summary>
        /// מספר קטלוגי, עד 50 תווים
        /// </summary>
        [DataMember, StringLength(50)]
        public virtual string CatalogNumber { get; set; }

        /// <summary>
        /// מספר קטלוגי של הספק/יצרן
        /// </summary>
        [DataMember, StringLength(50)]
        public virtual string SupplierCatalogNumber1 { get; set; }
        [DataMember, StringLength(50)]
        public virtual string SupplierCatalogNumber2 { get; set; }
        [DataMember, StringLength(50)]
        public virtual string SupplierCatalogNumber3 { get; set; }

        /// <summary>
        /// סטטוס - true = פעיל, false = לא פעיל
        /// </summary>
        [DataMember]
        public virtual bool Status { get; set; }

        /// <summary>
        /// אחריות למוצר. מודפסת בהצעות מחיר ובחשבוניות בכספית+מלאי
        /// </summary>
        [DataMember, StringLength(1000)]
        public virtual string Warranty { get; set; }

        /// <summary>
        /// מחיר רכישה של הפריט
        /// </summary>
        [DataMember]
        public virtual decimal? PurchasePrice { get; set; }

        /// <summary>
        /// מחיר מכירה של הפריט - מחירון מספר 1
        /// </summary>
        [DataMember]
        public virtual decimal? SalePrice1 { get; set; }
        /// <summary>
        /// מחיר מכירה של הפריט - מחירון מספר 2
        /// </summary>
        [DataMember]
        public virtual decimal? SalePrice2 { get; set; }
        /// <summary>
        /// מחיר מכירה של הפריט - מחירון מספר 3
        /// </summary>
        [DataMember]
        public virtual decimal? SalePrice3 { get; set; }
        /// <summary>
        /// מחיר מכירה של הפריט - מחירון מספר 4
        /// </summary>
        [DataMember]
        public virtual decimal? SalePrice4 { get; set; }

        /// <summary>
        /// האם לגבות מע"מ על הפריט. לדוגמה: אין מעמ על ירקות ופירות.
        /// </summary>
        [DataMember]
        public virtual bool? ChargeVat { get; set; }

        /// <summary>
        /// סמל המטבע שבו נרכש ונמכר הפריט - הסמלים האפשריים הם הסמלים שטבלת המטבעות של כספית.
        /// ראה במסך הראשי, מסך מטבעות. לדוגמה: ₪, $, €, A$
        /// </summary>
        [DataMember, StringLength(4)]
        public string CurrencySymbol { get; set; }

        /// <summary>
        /// מספר יחידות שנמצאות במלאי. כספית מחסרת את הכמות שנמכרה בעת מכירה של הפריט בחשבונית או בתעודת משלוח
        /// </summary>
        [DataMember]
        public virtual decimal? QtyInStock { get; set; }
        /// <summary>
        /// מספר יחידות מזערי במלאי. מתחת למספר זה כספית תציג התראה בעת מכירה של הפריט בחשבונית או בתעודת משלוח
        /// </summary>
        [DataMember]
        public virtual decimal? MinQtyInStock { get; set; }

        /// <summary>
        /// מספר יחידות מירבי במלאי. מתחת למספר זה כספית תציג התראה בעת מכירה של הפריט בחשבונית או בתעודת משלוח
        /// </summary>
        [DataMember]
        public virtual decimal? MaxQtyInStock { get; set; }

        /// <summary>
        /// שם היחידה: ק"ג , יח', קופסה
        /// לא בשימוש.
        /// </summary>
        [DataMember, StringLength(20)]
        public virtual string UnitName { get; set; }

        /// <summary>
        /// שם המחלקה/הקבוצה שלה שייך הפריט
        /// </summary>
        [DataMember, StringLength(40)]
        public virtual string Class { get; set; }

        /// <summary>
        /// האם המחירים של הפריט כוללים מעמ. 
        /// אם כן, כספית תחלץ את המחיר לפני מעמ כשהמשתמש בוחר בפריט בחשבונית.
        /// </summary>
        [DataMember]
        public virtual bool IsPriceIncludeVat { get; set; }
    }
}
