﻿using CaspitWebTiles.Api.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml.Linq;

namespace ApiTester
{
    partial class frmMain
    {
        private string GetAllProducts()
        {
            string url = string.Format("{0}/api/v1/products/?token={1}",
                txtUrl.Text, txtToken.Text);

            return SendGetRequest(url);
        }

        private void InitProductList(ComboBox c)
        {
            List<ValueNamePair<string>> v = new List<ValueNamePair<string>>();
            v.Add(new ValueNamePair<string>(null, "--בחר--"));

            string msg = GetAllProducts();

            var doc = XDocument.Parse(msg);

            XNamespace ns = "http://www.caspit.biz/2013/03";
            var products = doc.Root.Descendants(ns + "CaspitBase");

            foreach (XElement product in products)
            {
                var productName = product.Element(ns + "Name").Value;
                var key = product.Element(ns + "ProductId").Value;
                v.Add(new ValueNamePair<string>(key, productName));
            }

            c.DataSource = v;
            c.DisplayMember = "Name";
            c.ValueMember = "Value";
            c.SelectedIndex = 0;
        }

        private void btnNewProduct_Click(object sender, EventArgs e)
        {
            try
            {
                string url;
                CaspitProduct product;

                if (lstProductCurrency.SelectedValue == null)
                {
                    MessageBox.Show("חובה לבחור במטבע");
                    return;
                }
                PrepareDocumentMessage(out url, out product, txtDocumentKey.Text);

                string content = SendPostRequest<CaspitProduct>(url, product);

                MessageBox.Show("Operation completed OK. Data returned:\r\n" + content, "", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnUpdateProduct_Click(object sender, EventArgs e)
        {
            try
            {
                string url;
                CaspitProduct product;

                if (lstProductCurrency.SelectedValue == null)
                {
                    MessageBox.Show("חובה לבחור במטבע");
                    return;
                }
                PrepareDocumentMessage(out url, out product, txtDocumentKey.Text);

                string content = SendPostRequest<CaspitProduct>(url, product);

                MessageBox.Show("Operation completed OK. Data returned:\r\n" + content, "", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="url"></param>
        /// <param name="product"></param>
        /// <param name="id">The id WE (=the client) use to identify the document. It is NOT the document number in Caspit.</param>
        private void PrepareDocumentMessage(out string url, out CaspitProduct product, string id)
        {
            //  .../api/v1/documents/2a193514-e412-21b6-a97b-5f0ae26ad332?token=4b193514-e412-41b7-a97b-5f0ae26ad351
            url = string.Format("{0}/api/v1/Products/{2}?token={1}",
                txtUrl.Text, txtToken.Text, id);

            product = new CaspitProduct
            {
                ProductId = txtProductKey.Text,
                CatalogNumber = txtProductCatalogNumber.Text,
                Class = txtProductClass.Text,
                Description = txtProductDesciption.Text,
                Name = txtProductName.Text,
                SalePrice1 = string.IsNullOrEmpty(txtProductPrice1.Text) ? (decimal?)null : decimal.Parse(txtProductPrice1.Text),
                PurchasePrice = string.IsNullOrEmpty(txtProductPurchasePrice.Text) ? (decimal?)null : decimal.Parse(txtProductPurchasePrice.Text),
                CurrencySymbol = lstProductCurrency.SelectedValue.ToString()
            };
        }
    }
}
