﻿/*
 *  Resources:
 *  http://www.asp.net/web-api/overview/web-api-clients/calling-a-web-api-from-a-net-client
 *  http://www.agile-code.com/blog/building-an-asp-net-web-api-restful-service/
 */
using System;
using System.Windows.Forms;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Net;
using System.IO;
using System.Linq;
using System.Xml.Linq;
using CaspitWebTiles.Api.Models;
using System.Net.Http.Formatting;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Threading.Tasks;
using Microsoft.Win32;

namespace ApiTester
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Get a new Token from Caspit Web
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetToken_Click(object sender, EventArgs e)
        {
            try
            {
                WriteRegsitryKey("UserName", txtUserName.Text);
                WriteRegsitryKey("Password", txtPassword.Text);
                WriteRegsitryKey("OsekMorshe", txtOsekMorshe.Text);
                WriteRegsitryKey("Url", txtUrl.Text);

                GetToken();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void GetToken()
        {
            //  .../api/v1/token?user=demo&pwd=demodemo&osekmorshe=098765432
            string url = string.Format("{0}/api/v1/token/?user={1}&pwd={2}&osekmorshe={3}", txtUrl.Text,
                txtUserName.Text, txtPassword.Text, txtOsekMorshe.Text);

            string content = SendGetRequest(url);

            var doc = XDocument.Parse(content);
            var ns = (XNamespace)"http://schemas.microsoft.com/2003/10/Serialization/";
            txtToken.Text = doc.Element(ns + "string").Value;
            txtTokenDate.Text = SystemTime.Now().ToShortTimeString();        
        }

        #region Contacts
        private void btnAllContacts_Click(object sender, EventArgs e)
        {
            try
            {
                string url = string.Format("{0}/api/v1/contacts/?token={1}",
                    txtUrl.Text, txtToken.Text);

                string content = SendGetRequest(url);

                txtAllContacts.Text = content;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #region /////////////////// Create a new CaspitContact ///////////////
        private void btnNewContact_Click(object sender, EventArgs e)
        {
            try
            {
                string url;
                CaspitContact contact;

                PrepareContactMessage(out url, out contact, null);

                string content = SendPostRequest <CaspitContact>(url, contact);

                MessageBox.Show("Operation completed OK. Data returned:\r\n" + content, "", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        
        private void btnUpdateContact_Click(object sender, EventArgs e)
        {
            try
            {
                string url;
                CaspitContact contact;

                PrepareContactMessage(out url, out contact, txtClientKey.Text);

                string content = SendPutRequest<CaspitContact>(url, contact);

                MessageBox.Show("Operation completed OK. Data returned:\r\n" + content, "", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

         private void btnFindContact_Click(object sender, EventArgs e)
         {
             try
             {
                 string url = string.Format("{0}/api/v1/contacts/{2}?token={1}",
                     txtUrl.Text, txtToken.Text, txtClientKey.Text);
                 GetClient(url, lstResponseFormat.Text);
             }
            catch (Exception ex)
             {
                 MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
         }

         private void btnFindCustomerOsekMorshe_Click(object sender, EventArgs e)
         {
             try
             {
                 string url = string.Format("{0}/api/v1/contacts/?token={1}&osekMorshe={2}&d=1",
                     txtUrl.Text, txtToken.Text, txtClientOsekMorshe.Text);
                 GetClient(url, lstResponseFormat.Text);
             }
             catch (Exception ex)
             {
                 MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
             }
         }

        private void GetClient(string url, string format = "xml")
        {
            try
            {
                 string data = SendGetRequest(url, format: format);

                 var doc = XDocument.Parse(data);

                 if (doc.Root.Name.LocalName != "CaspitContact")
                 {
                     throw new ApplicationException("Unexpected result. Expected: CaspitContact, Received: " + doc.Root.Name.LocalName);
                 }

                 XNamespace ns = "http://www.caspit.biz/2013/03";

                 txtClientCaspitNumber.Text = doc.Root.Element(ns + "Number").Value;
                 txtClientBusinessName.Text = doc.Root.Element(ns + "BusinessName").Value;
                 txtClientContactName.Text = doc.Root.Element(ns + "Name").Value;
                 txtClientAddress1.Text = doc.Root.Element(ns + "Address1").Value;
                 txtClientAddress2.Text = doc.Root.Element(ns + "Address2").Value;
                 txtClientCity.Text = doc.Root.Element(ns + "City").Value;
                 txtClientPostalCode.Text = doc.Root.Element(ns + "PostalCode").Value;
                 txtClientPhone.Text = doc.Root.Element(ns + "Phone").Value;
                 txtClientMobile.Text = doc.Root.Element(ns + "Mobile").Value;
                 txtClientMail.Text = doc.Root.Element(ns + "Email").Value;
                 txtClientOpenBalance.Text = doc.Root.Element(ns + "OpenBalance").Value;
                 txtClientBalance.Text = doc.Root.Element(ns + "Balance").Value;
                 txtClientBalanceIska.Text = doc.Root.Element(ns + "BalanceInvoiceIska").Value;
                 lstContactType.SelectedValue = int.Parse(doc.Root.Element(ns + "ContactType").Value);
                 if (string.IsNullOrEmpty(doc.Root.Element(ns + "TrxCodeNumber").Value))
                 {
                     lstTrxCodes.SelectedIndex = 0;
                 }
                 else
                 {
                     lstTrxCodes.SelectedValue = int.Parse(doc.Root.Element(ns + "TrxCodeNumber").Value);
                 }
                 txtComment1.Text = doc.Root.Element(ns + "Comments1").Value;
                 txtComment2.Text = doc.Root.Element(ns + "Comments2").Value;
                 txtComment3.Text = doc.Root.Element(ns + "Comments3").Value;
                 txtInvoiceComment.Text = doc.Root.Element(ns + "InvoiceComment").Value;

                 if (string.IsNullOrEmpty(doc.Root.Element(ns + "BankAcctBankId").Value))
                 {
                     lstClientBankNumber.SelectedIndex = 0;
                 }
                 else
                 {
                     lstClientBankNumber.SelectedValue = int.Parse(doc.Root.Element(ns + "BankAcctBankId").Value);
                 }
                 txtClientBankBranch.Text = doc.Root.Element(ns + "BankAcctBranch").Value;
                 txtClientBankAccount.Text = doc.Root.Element(ns + "BankAcctNumber").Value;
             }
             catch (Exception ex)
             {
                 throw new Exception("", ex);
             }
         }

        private void PrepareContactMessage(out string url, out CaspitContact contact, string id)
         {
             //  .../api/v1/contacts/46467?token=4b193514-e412-41b7-a97b-5f0ae26ad351
             url = string.Format("{0}/api/v1/contacts/{2}?token={1}",
                 txtUrl.Text, txtToken.Text, id);

             contact = new CaspitContact
             {
                 Number = ZI(txtClientCaspitNumber.Text),
                 ContactId = Z(txtClientKey.Text),
                 BusinessName = Z(txtClientBusinessName.Text),
                 Name = Z(txtClientContactName.Text),
                 Address1 = Z(txtClientAddress1.Text),
                 Address2 = Z(txtClientAddress2.Text),
                 City = Z(txtClientCity.Text),
                 PostalCode = Z(txtClientPostalCode.Text),
                 Email = Z(txtClientMail.Text),
                 Phone = Z(txtClientPhone.Text),
                 Mobile = Z(txtClientMobile.Text),
                 TrxCodeNumber = lstTrxCodes.SelectedItem == null ? null : ((ValueNamePair<int?>)lstTrxCodes.SelectedItem).Value,
                 OpenBalance = ZI(txtClientOpenBalance.Text),
                 ContactType = lstContactType.SelectedItem == null ? null : ((ValueNamePair<int?>)lstContactType.SelectedItem).Value,
                 Comments1 = txtComment1.Text,
                 Comments2 = txtComment2.Text,
                 Comments3 = txtComment3.Text,
                 InvoiceComment = txtInvoiceComment.Text,
                 BankAcctBankId = lstClientBankNumber.SelectedItem == null ? null : ((ValueNamePair<int?>)lstClientBankNumber.SelectedItem).Value,
                 BankAcctBranch = txtClientBankBranch.Text,
                 BankAcctNumber = txtClientBankAccount.Text
             };
         }
        #endregion
        #endregion

        #region //////////////////// Documents //////////////////////////////
        private void btnAllDocuments_Click(object sender, EventArgs e)
        {
            try
            {
                string url = string.Format("{0}/api/v1/documents/?token={1}",
                    txtUrl.Text, txtToken.Text);

                string content = SendGetRequest(url);

                txtAllDocuments.Text = content;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        #endregion



        #region /////////// Products ///////////////

        private void btnAllProducts_Click(object sender, EventArgs e)
        {
            try
            {
                string url = string.Format("{0}/api/v1/products/?token={1}",
                    txtUrl.Text, txtToken.Text);

                string content = SendGetRequest(url);

                txtAllProducts.Text = content;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnCleanProduct_Click(object sender, EventArgs e)
        {
            CleanProduct();
        }

        private void CleanProduct()
        {
            txtProductCatalogNumber.Text = null;
            txtProductClass.Text = null;
            txtProductDesciption.Text = null;
            txtProductPrice1.Text = null;
            txtProductPurchasePrice.Text = null;
            lstProductCurrency.SelectedIndex = 0;
        }

        private void btnFindProduct_Click(object sender, EventArgs e)
        {
            try
            {
                CleanProduct();

                string url = string.Format("{0}/api/v1/products/{2}?token={1}",
                    txtUrl.Text, txtToken.Text, txtProductKey.Text.Replace('/', ' '));
                string data = SendGetRequest(url);

                var doc = XDocument.Parse(data);

                if (doc.Root.Name.LocalName != "CaspitProduct")
                {
                    throw new ApplicationException("Unexpected result. Expected: CaspitProduct, Received: " + doc.Root.Name.LocalName);
                }

                XNamespace ns = "http://www.caspit.biz/2013/03";
                txtProductName.Text = doc.Root.Element(ns + "Name").Value;
                txtProductDesciption.Text = doc.Root.Element(ns + "Description").Value;
                txtProductClass.Text = doc.Root.Element(ns + "Class").Value;
                lstProductCurrency.SelectedValue = doc.Root.Element(ns + "CurrencySymbol").Value;
                txtProductPrice1.Text = doc.Root.Element(ns + "SalePrice1").Value;
                txtProductCatalogNumber.Text = doc.Root.Element(ns + "CatalogNumber").Value;
                txtProductPurchasePrice.Text = doc.Root.Element(ns + "PurchasePrice").Value;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }        
        
       
        #endregion

        #region ///////////// Some infrastructure /////////////
        private string SendGetRequest(string url, string method = "GET", string format = "xml")
        {
            //Please be patient ...
            this.Cursor = Cursors.WaitCursor;

            var client = PrepareHttpClient(format);
            txtSent.Text = url;

            HttpRequestMessage request = new HttpRequestMessage(new HttpMethod(method), url);

            var taskReult = client.SendAsync(request);
            HttpResponseMessage httpResponse = taskReult.Result;

            Stream st = httpResponse.Content.ReadAsStreamAsync().Result;
            StreamReader reader = new StreamReader(st);
            string response = reader.ReadToEnd();

            //Reset the cursor shape
            this.Cursor = Cursors.Default;

            txtSent.Text = FormatRequest(httpResponse.RequestMessage, "");
            txtReceived.Text = FormatResponse(httpResponse, response);

            //For GET we expect a response of 200 OK
            if (httpResponse.StatusCode == HttpStatusCode.OK)
            {
                return response;
            }
 
            throw new ApplicationException(response);
        }

        /// <summary>
        /// Post to the server using JSON
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="url">The server uri</param>
        /// <param name="dataSentToServer">The object to POST</param>
        /// <returns></returns>
        private string SendPostRequest<T>(string url, T dataSentToServer, string method = "POST", 
            string acceptFormat = "xml")
        {
            this.Cursor = Cursors.WaitCursor;
            HttpClient client = PrepareHttpClient(acceptFormat);

            HttpContent content;

            if (typeof(T) != typeof(string))
            {
                // Use the JSON formatter to create the content of the request body.
                content = new ObjectContent<T>(dataSentToServer, new JsonMediaTypeFormatter());
            }
            else
            {
                content = new StringContent(dataSentToServer.ToString());
            }

            Stream st = content.ReadAsStreamAsync().Result;
            StreamReader reader = new StreamReader(st);
            string s = reader.ReadToEnd();

            HttpRequestMessage request = new HttpRequestMessage(new HttpMethod(method), url);
            request.Content = content;

            // Send the request.
            //var taskResult = client.PostAsync(url, content);

            var taskResult = client.SendAsync(request);


            //Note: We could simply perform the following line and save some time
            //but then we will not have access to the post content:
            //var taskResult = client.PostAsJsonAsync<T>(url, e);
           
            HttpResponseMessage httpResponse = taskResult.Result;
            this.Cursor = Cursors.Default;

            txtSent.Text = FormatRequest(httpResponse.RequestMessage, s);

            st = httpResponse.Content.ReadAsStreamAsync().Result;
            reader = new StreamReader(st);
            string responseContent = reader.ReadToEnd();
            txtReceived.Text = FormatResponse(httpResponse, responseContent);

            switch (method)
            {
                case "POST":
                    //For POST we expect a response of 201 Created
                    if (httpResponse.StatusCode == HttpStatusCode.Created || httpResponse.StatusCode == HttpStatusCode.NoContent)
                    {
                        return responseContent;
                    }
                    break;
                //For PUT (update) we expect a response of 200 OK
                case "PUT":
                //For DELETE we expect a response of 200 OK
                case "DELETE":
                default:
                    if (httpResponse.StatusCode == HttpStatusCode.OK)
                    {
                        return responseContent;
                    }
                    break;
            }


            throw new ApplicationException(responseContent);
        }

        /// <summary>
        /// PUT (=update) to the server using JSON
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="url"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        private string SendPutRequest<T>(string url, T e)
        {
            return SendPostRequest<T>(url, e, "PUT");
        }

        private string SendDeleteRequest<T>(string url, T e)
        {
            return SendPostRequest<T>(url, e, "DELETE");
        }

        private string SendEmailRequest<T>(string url, T e)
        {
            return SendPostRequest<T>(url, e, "EMAIL");
        }

        private string FormatRequest(HttpRequestMessage request, string content)
        {
            return
                string.Format("{0} {1} HTTP/{2}\r\n{3}\r\n{4}",
                    request.Method,
                    request.RequestUri,
                    request.Version,
                    request.Headers,
                    content);
        }

        private string FormatResponse(HttpResponseMessage result, string responseText)
        {
            return
                string.Format("HTTP/{0} {1} {2}\r\n{3}\r\n{4}",
                    result.Version,
                    (int)result.StatusCode,
                    result.ReasonPhrase,
                    result.Headers,
                    responseText);
        }

        private HttpClient PrepareHttpClient(string accept)
        {
            var client = new HttpClient();

            switch (accept)
            {
                default:
                case "xml":
                    client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/xml"));
                    break;
                case "json":
                    client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
                    break;
            }

            return client;
        }
        #endregion

        #region //////////////// Initialization ///////////////////////
        private void frmMain_Load(object sender, EventArgs e)
        {

            List<ValueNamePair<int?>> v = new List<ValueNamePair<int?>>();
            v.Add(new ValueNamePair<int?>(null, "--בחר--"));
            v.Add(new ValueNamePair<int?>(2, "קבלות"));
            v.Add(new ValueNamePair<int?>(3, "מכירות"));
            v.Add(new ValueNamePair<int?>(4, "מכירות פטורות ממע\"מ"));

            lstTrxCodes.DataSource = v;
            lstTrxCodes.DisplayMember = "Name";
            lstTrxCodes.ValueMember = "Value";
            lstTrxCodes.SelectedIndex = 0;

            lstDocumentTrxCodes.DataSource = v;
            lstDocumentTrxCodes.DisplayMember = "Name";
            lstDocumentTrxCodes.ValueMember = "Value";
            lstDocumentTrxCodes.SelectedIndex = 0;

            v = new List<ValueNamePair<int?>>();
            v.Add(new ValueNamePair<int?>(null, "--בחר--"));
            v.Add(new ValueNamePair<int?>(0, "לקוח"));
            v.Add(new ValueNamePair<int?>(1, "ספק"));
            v.Add(new ValueNamePair<int?>(2, "לקוח וגם ספק"));
            lstContactType.DataSource = v;
            lstContactType.DisplayMember = "Name";
            lstContactType.ValueMember = "Value";
            lstContactType.SelectedIndex = 0;


            InitBankList(lstClientBankNumber);

            //Document - Numbers taken from Globals.TrxTypes
            v = new List<ValueNamePair<int?>>();
            v.Add(new ValueNamePair<int?>(null, "--בחר--"));
            v.Add(new ValueNamePair<int?>(1, "חשבונית"));
            v.Add(new ValueNamePair<int?>(7, "חש/קבלה"));
            v.Add(new ValueNamePair<int?>(2, "קבלה"));
            v.Add(new ValueNamePair<int?>(6, "חש/עסקה"));
            v.Add(new ValueNamePair<int?>(15, "תע משלוח"));

            lstDocumentTrxType.DataSource = v;
            lstDocumentTrxType.DisplayMember = "Name";
            lstDocumentTrxType.ValueMember = "Value";
            lstDocumentTrxType.SelectedIndex = 0;

            InitCurrencyList(lstDocumentLineCurrency1);
            InitCurrencyList(lstDocumentLineCurrency2);
            InitCurrencyList(lstDocumentLineCurrency3);

            InitBankList(lstReceiptLineBank1);
            InitBankList(lstReceiptLineBank2);

            InitCurrencyList(lstReceiptCurrency);

            InitPmtTypeList(lstReceiptLinePmtType1);
            InitPmtTypeList(lstReceiptLinePmtType2);

            InitCurrencyList(lstProductCurrency);

            InitDocumentStatusList(lstDocumentStatus);

            InitServerLogin();

        }

        bool initDone = false;
        /// <summary>
        /// Init the product list in the Docment tab
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tabControl1_Selected(object sender, TabControlEventArgs e)
        {
            try
            {
                if (!initDone)
                {
                    if (string.IsNullOrEmpty(txtToken.Text))
                    {
                        GetToken();
                    }
                    InitProductList(lstDocumentLineProduct1);
                    InitProductList(lstDocumentLineProduct2);
                    InitProductList(lstDocumentLineProduct3);
                    initDone = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        #endregion

        private void btnFillForm_Click(object sender, EventArgs e)
        {
            try
            {
                Random rnd = new Random((int)DateTime.Now.Ticks);
                txtClientKey.Text = rnd.Next(5001, 9999).ToString();
                txtClientOsekMorshe.Text = string.Format("{0:00000}{1:00000}",
                    rnd.Next(00500, 99999).ToString(),
                    rnd.Next(10000, 99999).ToString()
                    );
                txtClientBusinessName.Text = string.Format("{0}",
                    new string[] { "אלי אלומיניום", "אחים חורב", "מוסך פז'ו צפת", "המייסדים בע\"מ", "להב נשק בע\"מ" }[rnd.Next(0, 4)]);
                txtClientContactName.Text = string.Format("{0} {1}",
                    new string[] { "דרור", "ירון", "יעקב", "לאה", "ציפי" }[rnd.Next(0, 4)],
                    new string[] { "בז'רנו", "לחמי", "רצין", "כ\"ץ", "אסולין" }[rnd.Next(0, 4)]
                    );

                txtClientAddress1.Text = string.Format("{0}",
                    new string[] { "בן גוריון 88", "מרכז חורב", "דרך המכבים 51 ", "תחנת דלק פז", "אליהו הנביא 5א'" }[rnd.Next(0, 4)]);
                txtClientAddress2.Text = string.Format("{0}",
                    new string[] { "בחצר", "ת.ד. 625", null, "כניסה צפונית", "קומה ב'" }[rnd.Next(0, 4)]);
                txtClientCity.Text = string.Format("{0}",
                    new string[] { "י-ם", "חיפה", null, "אילת", "ב\"ש" }[rnd.Next(0, 4)]);
                txtClientPostalCode.Text = rnd.Next(63732, 90728).ToString();
                txtClientMail.Text = new string[] { "pati@zoom.com", "tal@zami.co.il", null, "r.co@oper.biz", "qal90@gmail.com" }[rnd.Next(0, 4)];
                txtClientMobile.Text = string.Format("05{0}-{1:0000000}", rnd.Next(1, 7), rnd.Next(1111111, 9999999));
                txtClientPhone.Text = string.Format("0{0}-{1:0000000}", rnd.Next(2, 99), rnd.Next(1111111, 9999999));
                txtClientCaspitNumber.Text = rnd.Next(1111111, 9999999).ToString();

                lstClientBankNumber.SelectedIndex = rnd.Next(1, 4);
                txtClientBankBranch.Text = rnd.Next(355, 9999).ToString();
                txtClientBankAccount.Text = rnd.Next(3558657, 9999999).ToString();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnCleanCustomer_Click(object sender, EventArgs e)
        {
            try
            {
                txtClientKey.Text = null;
                txtClientCaspitNumber.Text = null;
                txtClientOsekMorshe.Text = null;
                txtClientBusinessName.Text = null;
                txtClientContactName.Text = null;
                txtClientContactName.Text = null;
                txtClientAddress1.Text = null;
                txtClientAddress2.Text = null;
                txtClientCity.Text = null;
                txtClientPostalCode.Text = null;
                txtClientMail.Text = null;
                txtClientMobile.Text = null;
                txtClientPhone.Text = null;
                txtClientBankBranch.Text = null;
                txtClientBankAccount.Text = null;
                lstClientBankNumber.SelectedIndex = 0;
                lstTrxCodes.SelectedIndex = 0;
                lstContactType.SelectedIndex = 0;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        #region Utils
        /// <summary>
        /// Return either the string or null
        /// </summary>
        /// <param name="s"></param>
        /// <returns></returns>
        private string Z(string s)
        {
            if (string.IsNullOrEmpty(s)) return null;

            return s;
        }
        /// <summary>
        /// Parse a string into int?
        /// </summary>
        /// <param name="s"></param>
        /// <returns></returns>
        private int? ZI(string s)
        {
            if (string.IsNullOrEmpty(s)) return null;

            return int.Parse(s);
        }
        /// <summary>
        /// Get a formatted Currency value
        /// </summary>
        /// <param name="e"></param>
        /// <param name="ns"></param>
        /// <param name="n"></param>
        /// <returns></returns>
        private string GetCValue(XElement e, XNamespace ns, string n)
        {
            return string.Format("{0:#,##0.00}", decimal.Parse(e.Element(ns + n).Value));
        }

        #endregion

        private void btnAllExpenses_Click(object sender, EventArgs e)
        {
            try
            {
                string url = string.Format("{0}/api/v1/Expenses/?token={1}",
                    txtUrl.Text, txtToken.Text);

                string content = SendGetRequest(url);

                tabAllExpenses.Text = content;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void InitServerLogin()
        {
            txtUserName.Text = ReadRegsitryKey("UserName");
            txtUrl.Text = ReadRegsitryKey("Url");
            txtPassword.Text = ReadRegsitryKey("Password");
            txtOsekMorshe.Text = ReadRegsitryKey("OsekMorshe");
        }

        RegistryKey baseRegistryKey = Registry.CurrentUser;
        string subKey = "Software\\VB and VBA Program Settings\\CaspitWeb Api Tester";

        public string ReadRegsitryKey(string KeyName)
        {
            // Opening the registry key
            RegistryKey rk = baseRegistryKey;
            // Open a subKey as read-only
            RegistryKey sk1 = rk.OpenSubKey(subKey);
            // If the RegistrySubKey doesn't exist -> (null)
            if (sk1 == null)
            {
                return null;
            }
            else
            {
                try
                {
                    // If the RegistryKey exists I get its value
                    // or null is returned.
                    return (string)sk1.GetValue(KeyName);
                }
                catch
                {
                    throw;
                }
            }
        }

        public bool WriteRegsitryKey(string KeyName, object Value)
        {
            try
            {
                // Setting
                RegistryKey rk = baseRegistryKey;
                // I have to use CreateSubKey 
                // (create or open it if already exits), 
                // 'cause OpenSubKey open a subKey as read-only
                RegistryKey sk1 = rk.CreateSubKey(subKey);
                // Save the value
                sk1.SetValue(KeyName, Value);

                return true;
            }
            catch 
            {
                throw;
            }
        }

        private void btnBillGold_Click(object sender, EventArgs e)
        {
            string url = string.Format("{0}/api/v1/BillGoldDocIsReady/", txtUrl.Text);

            string data = string.Format("BillGoldCompID={0}&DocType={1}&DocNumber={2}&ExternalCompID={3}",
                txtBillGoldCompId.Text, txtBillGoldDocType.Text, txtBillGoldDocNumber.Text, txtBillGoldExternalCompId.Text);

            string content = SendPostRequest<string>(url, data);
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            try
            {
                string url = string.Format("{0}/api/v1/contacts/{2}?token={1}",
                     txtUrl.Text, txtToken.Text, txtClientKey1.Text);

                string content = SendDeleteRequest<CaspitContact>(url, null);

                MessageBox.Show("Operation completed OK. Data returned:\r\n" + content, "", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }

    class MessageHandlerForPrint : DelegatingHandler
    {
        public MessageHandlerForPrint(HttpMessageHandler innerHandler) : base(innerHandler)
        {

        }
        protected override Task<HttpResponseMessage> SendAsync(
            HttpRequestMessage request, System.Threading.CancellationToken cancellationToken)
        {
            request.Method = new HttpMethod("PRINT");
            
            return base.SendAsync(request, cancellationToken);
        }
    }

    class LoggingHandler : DelegatingHandler
    {
        StreamWriter _writer;

        public LoggingHandler(Stream stream)
        {
            _writer = new StreamWriter(stream);
        }

        protected override Task<HttpResponseMessage> SendAsync(
            HttpRequestMessage request, System.Threading.CancellationToken cancellationToken)
        {
            Task<HttpResponseMessage> response = base.SendAsync(request, cancellationToken);

            if (response.Status == TaskStatus.RanToCompletion)
            {
                _writer.WriteLine("{0}\t{1}\t{2}", request.RequestUri,
                    (int)response.Result.StatusCode, response.Result.Headers.Date);
            }
            return response;
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                _writer.Dispose();
            }
            base.Dispose(disposing);
        }
    }

}
